@@ -284,13 +284,14 @@ process_media_file(char *file, time_t mtime, off_t size, int compilation)
   char *filename;
   char *ext;
   time_t stamp;
+  int id;
   int ret;
 
-  stamp = db_file_stamp_bypath(file);
+  db_file_stamp_bypath(file, &stamp, &id);
 
   if (stamp >= mtime)
     {
-      db_file_ping(file);
+      db_file_ping(id);
       return;
     }
 
@@ -787,6 +788,8 @@ bulk_scan(void)
 					   DPRINTF(E_DBG, L_SCAN, "Purging old database content\n");
 					   db_purge_cruft(start);
 
+					   db_hook_post_scan();
+
 					   db_pool_release();
 
 					   dispatch_group_leave(scanner_grp);
