@@ -58,6 +58,8 @@
 
 /* Session timeout in seconds */
 #define DAAP_SESSION_TIMEOUT 1800
+/* Update requests refresh interval in seconds */
+#define DAAP_UPDATE_REFRESH  300
 
 
 struct uri_map {
@@ -77,6 +79,9 @@ struct daap_update_request {
   struct http_request *req;
   struct http_response *r;
 
+  /* Refresh tiemout */
+  dispatch_source_t timeout;
+
   struct daap_update_request *next;
 };
 
@@ -100,6 +105,7 @@ static avl_tree_t *daap_sessions;
 static int next_session_id;
 
 /* Update requests */
+static int current_rev;
 static dispatch_queue_t updates_sq;
 static struct daap_update_request *update_requests;
 
@@ -128,10 +134,8 @@ daap_session_free(void *item)
 
   s = (struct daap_session *)item;
 
-#if 0
   dispatch_source_cancel(s->timeout);
   dispatch_release(s->timeout);
-#endif /* 0 */
   free(s);
 }
 
@@ -147,7 +151,6 @@ daap_session_kill(struct daap_session *s)
       });
 }
 
-#if 0
 /* Queue: sessions_sq */
 static void
 daap_session_timeout_cb(void *arg)
@@ -160,7 +163,6 @@ daap_session_timeout_cb(void *arg)
 
   daap_session_kill(s);
 }
-#endif /* 0 */
 
 /* Queue: HTTP read queue */
 static struct daap_session *
@@ -179,7 +181,6 @@ daap_session_register(void)
 
   memset(s, 0, sizeof(struct daap_session));
 
-#if 0
   s->timeout = dispatch_source_create(DISPATCH_SOURCE_TYPE_TIMER, 0, 0, sessions_sq);
   if (!s->timeout)
     {
@@ -195,7 +196,6 @@ daap_session_register(void)
   dispatch_source_set_timer(s->timeout,
 			    dispatch_time(DISPATCH_TIME_NOW, DAAP_SESSION_TIMEOUT * NSEC_PER_SEC),
 			    DISPATCH_TIME_FOREVER /* one-shot */, 30 * NSEC_PER_SEC);
-#endif /* 0 */
 
   dispatch_sync(sessions_sq, ^{
       avl_node_t *node;
@@ -213,9 +213,7 @@ daap_session_register(void)
 	  return;
 	}
 
-#if 0
       dispatch_resume(s->timeout);
-#endif
 
       b_ret = 0;
     });
@@ -265,11 +263,9 @@ daap_session_find(struct httpd_hdl *h, struct daap_session **s)
 
       *s = (struct daap_session *)node->item;
 
-#if 0
       dispatch_source_set_timer((*s)->timeout,
 				dispatch_time(DISPATCH_TIME_NOW, DAAP_SESSION_TIMEOUT * NSEC_PER_SEC),
 				DISPATCH_TIME_FOREVER /* one-shot */, 30 * NSEC_PER_SEC);
-#endif /* 0 */
     });
 
   if (!*s)
@@ -285,13 +281,10 @@ daap_session_find(struct httpd_hdl *h, struct daap_session **s)
 /* Update requests helpers */
 /* Queue: updates_sq */
 static void
-update_free_cb_task(void *arg)
+update_remove(struct daap_update_request *ur)
 {
-  struct daap_update_request *ur;
   struct daap_update_request *p;
 
-  ur = (struct daap_update_request *)arg;
-
   if (ur == update_requests)
     update_requests = ur->next;
   else
@@ -307,10 +300,83 @@ update_free_cb_task(void *arg)
 
       p->next = ur->next;
     }
+}
+
+static void
+update_free(struct daap_update_request *ur)
+{
+  dispatch_source_cancel(ur->timeout);
+  dispatch_release(ur->timeout);
 
   free(ur);
 }
 
+/* Queue: updates_sq */
+static void
+update_refresh_cb(void *arg)
+{
+  struct daap_update_request *ur;
+  struct evbuffer *evbuf;
+  int ret;
+
+  ur = (struct daap_update_request *)arg;
+
+  evbuf = evbuffer_new();
+  if (!evbuf)
+    {
+      DPRINTF(E_LOG, L_DAAP, "Could not allocate evbuffer for DAAP update data\n");
+
+      return;
+    }
+
+  ret = evbuffer_expand(evbuf, 32);
+  if (ret < 0)
+    {
+      DPRINTF(E_LOG, L_DAAP, "Could not expand evbuffer for DAAP update data\n");
+
+      return;
+    }
+
+  /* Send back current revision */
+  dmap_add_container(evbuf, "mupd", 24);
+  dmap_add_int(evbuf, "mstt", 200);         /* 12 */
+  dmap_add_int(evbuf, "musr", current_rev); /* 12 */
+
+  ret = http_response_set_status(ur->r, HTTP_OK, "OK");
+  if (ret < 0)
+    {
+      DPRINTF(E_LOG, L_DAAP, "Could not set response status for DAAP update\n");
+
+      evbuffer_free(evbuf);
+      return;
+    }
+
+  http_response_set_body(ur->r, evbuf);
+
+  ret = http_server_response_thaw_and_run(ur->c, ur->r);
+  if (ret < 0)
+    {
+      DPRINTF(E_LOG, L_DAAP, "Could not thaw and run response to DAAP update request\n");
+
+      http_server_kill_connection(ur->c);
+    }
+
+  update_remove(ur);
+  update_free(ur);
+}
+
+/* Queue: updates_sq */
+static void
+update_free_cb_task(void *arg)
+{
+  struct daap_update_request *ur;
+
+  ur = (struct daap_update_request *)arg;
+
+  update_remove(ur);
+  update_free(ur);
+}
+
 static void
 update_free_cb(void *data)
 {
@@ -638,7 +704,7 @@ daap_reply_server_info(struct httpd_hdl *h, struct evbuffer *evbuf, char **uri)
   passwd = cfg_getstr(lib, "password");
   name = cfg_getstr(lib, "name");
 
-  len = 136 + strlen(name);
+  len = 157 + strlen(name);
 
   ret = evbuffer_expand(evbuf, len);
   if (ret < 0)
@@ -673,10 +739,8 @@ daap_reply_server_info(struct httpd_hdl *h, struct evbuffer *evbuf, char **uri)
   dmap_add_int(evbuf, "apro", apro); /* 12 */
   dmap_add_string(evbuf, "minm", name); /* 8 + strlen(name) */
 
-#if 0
   dmap_add_int(evbuf, "mstm", DAAP_SESSION_TIMEOUT); /* 12 */
   dmap_add_char(evbuf, "msal", 1);   /* 9 */
-#endif
 
   dmap_add_char(evbuf, "mslr", 1);   /* 9 */
   dmap_add_char(evbuf, "msau", (passwd) ? 2 : 0); /* 9 */
@@ -859,7 +923,6 @@ daap_reply_update(struct httpd_hdl *h, struct evbuffer *evbuf, char **uri)
   struct daap_session *s;
   struct daap_update_request *ur;
   const char *param;
-  int current_rev = 2;
   int reqd_rev;
   int ret;
 
@@ -923,6 +986,23 @@ daap_reply_update(struct httpd_hdl *h, struct evbuffer *evbuf, char **uri)
 
       return dmap_send_error(h, "mupd", "Out of memory");
     }
+  memset(ur, 0, sizeof(struct daap_update_request));
+
+  ur->timeout = dispatch_source_create(DISPATCH_SOURCE_TYPE_TIMER, 0, 0, updates_sq);
+  if (!ur->timeout)
+    {
+      DPRINTF(E_LOG, L_DAAP, "Could not create dispatch source for update timeout\n");
+
+      free(ur);
+      return dmap_send_error(h, "mupd", "Could not create timer");
+    }
+
+  dispatch_set_context(ur->timeout, ur);
+  dispatch_source_set_event_handler_f(ur->timeout, update_refresh_cb);
+
+  dispatch_source_set_timer(ur->timeout,
+			    dispatch_time(DISPATCH_TIME_NOW, DAAP_UPDATE_REFRESH * NSEC_PER_SEC),
+			    DISPATCH_TIME_FOREVER /* one-shot */, 30 * NSEC_PER_SEC);
 
   /* Freeze the request */
   http_server_response_freeze(h->c, h->r, update_free_cb, ur);
@@ -935,6 +1015,8 @@ daap_reply_update(struct httpd_hdl *h, struct evbuffer *evbuf, char **uri)
   dispatch_sync(updates_sq, ^{
       ur->next = update_requests;
       update_requests = ur;
+
+      dispatch_resume(ur->timeout);
     });
 
   return 0;
@@ -2663,6 +2745,7 @@ daap_init(void)
   int ret;
 
   next_session_id = 100; /* gotta start somewhere, right? */
+  current_rev = 2;
   update_requests = NULL;
 
   sessions_sq = dispatch_queue_create("org.forked-daapd.daap.sessions", NULL);
